/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sis.test;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.tudarmstadt.maki.simonstrator.api.component.sis.exception.SiSTypeNotFoundException;
import de.tudarmstadt.maki.simonstrator.api.component.sis.exception.UnknownComparator;
import de.tudarmstadt.maki.simonstrator.api.component.sis.util.SiSCondition;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class SiSConditionTest {

	@BeforeClass
	public static void setUpEnvironment() {
		//
	}

	@Before
	public void setUp() {
		//
	}

	@After
	public void tearDown() {
		//
	}

	@AfterClass
	public static void tearDownEnvironment() {
		//
	}

	/**
	 * Reflexivity
	 */
	@Test
	public void test001_equals_reflexivity() {

		try {
			SiSCondition cond = new SiSCondition("ENERGY_BATTERY_CAPACITY", ">=", 0.5);
			assertTrue(cond.equals(cond));

		} catch (SiSTypeNotFoundException | UnknownComparator e) {
			// TODO: Let test fail correctly
			assertTrue(false);
		}

	}

	/**
	 * Symmetricalness
	 */
	@Test
	public void test002_equals_symmetricalness() {

		try {

			SiSCondition cond1 = new SiSCondition("ENERGY_BATTERY_CAPACITY", ">=", 0.5);
			SiSCondition cond2 = new SiSCondition("ENERGY_BATTERY_CAPACITY", ">=", 0.5);
			assertTrue(cond1.equals(cond2));
			assertTrue(cond2.equals(cond1));


		} catch (SiSTypeNotFoundException | UnknownComparator e) {
			// TODO: Let test fail correctly
			assertTrue(false);
		}

	}

	/**
	 * Null
	 */
	@Test
	public void test003_equals_null() {

		try {
			SiSCondition cond = new SiSCondition("ENERGY_BATTERY_CAPACITY", ">=", 0.5);
			assertFalse(cond.equals(null));
		} catch (SiSTypeNotFoundException | UnknownComparator e) {
			// TODO: Let test fail correctly
		}

	}

	/**
	 * Null
	 */
	@Test
	public void test004_hash() {

		try {
			SiSCondition cond1 = new SiSCondition("ENERGY_BATTERY_CAPACITY", ">=", 0.5);
			SiSCondition cond2 = new SiSCondition("ENERGY_BATTERY_CAPACITY", ">=", 0.5);

			assertTrue(cond1.hashCode() == cond2.hashCode());

		} catch (SiSTypeNotFoundException | UnknownComparator e) {
			// TODO: Let test fail correctly
		}

	}
}
