/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sis.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.HashSet;
import java.util.Random;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import de.tudarmstadt.maki.simonstrator.api.common.graph.INodeID;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSInfoProperties;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSRequest;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSScope;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class SiSRequestTests {

	Random rand = new Random();

	@BeforeClass
	public static void setUpEnvironment() {
		//
	}

	@Before
	public void setUp() {
		//
	}

	@After
	public void tearDown() {
		//
	}

	@AfterClass
	public static void tearDownEnvironment() {
		//
	}

	@Test
	public void test001_nullRequest() {
		SiSRequest null_request = SiSRequest.NONE;

		assertEquals(0L, null_request.getTimeout());
		assertEquals(Long.MAX_VALUE, null_request.getMaxInformationAge());
		assertNull(null_request.getScope());
		assertNull(null_request.getSourceComponent());
	}

	@Test
	public void test002_requestCreationGlobal() {

		long timeout = rand.nextLong();
		long maxInformationAge = rand.nextLong();

		SiSRequest req = new SiSRequest(timeout, maxInformationAge, SiSScope.GLOBAL, this.getClass());

		assertEquals(timeout, req.getTimeout());
		assertEquals(maxInformationAge, req.getMaxInformationAge());
		assertEquals(SiSScope.GLOBAL, req.getScope());
		assertEquals(this.getClass(), req.getSourceComponent());
	}

	@Test
	public void test003_requestCreationLocal() {

		long timeout = rand.nextLong();
		long maxInformationAge = rand.nextLong();

		SiSRequest req = new SiSRequest(timeout, maxInformationAge, SiSScope.NODE_LOCAL, this.getClass());

		assertEquals(timeout, req.getTimeout());
		assertEquals(maxInformationAge, req.getMaxInformationAge());
		assertEquals(SiSScope.NODE_LOCAL, req.getScope());
		assertEquals(this.getClass(), req.getSourceComponent());
	}

	@Test
	public void test005_requestCreationSet() {

		long timeout = rand.nextLong();
		long maxInformationAge = rand.nextLong();

		HashSet<INodeID> receivers = new HashSet<>(3);
		receivers.add(INodeID.get("A"));
		receivers.add(INodeID.get("B"));
		receivers.add(INodeID.get("C"));

		SiSRequest req = new SiSRequest(timeout, maxInformationAge, SiSScope.SET, receivers, this.getClass());

		assertEquals(timeout, req.getTimeout());
		assertEquals(maxInformationAge, req.getMaxInformationAge());
		assertEquals(SiSScope.SET, req.getScope());
		assertEquals(this.getClass(), req.getSourceComponent());
		assertEquals(receivers, req.getReceivers());
	}

	@Test(expected = java.lang.AssertionError.class)
	public void test006_requestCreationExceptionGlobal() {
		new SiSRequest(0, 0, SiSScope.GLOBAL, new HashSet<>(), this.getClass());
	}

	@Test(expected = java.lang.AssertionError.class)
	public void test007_requestCreationExceptionLocal() {
		new SiSRequest(0, 0, SiSScope.NODE_LOCAL, new HashSet<>(), this.getClass());
	}

	@Test(expected = java.lang.AssertionError.class)
	public void test009_requestCreationExceptionSet() {
		new SiSRequest(0, 0, SiSScope.SET, this.getClass());
	}

	@Test
	public void test010_satisfiableBy() {

		long timeout = rand.nextLong();
		long maxInformationAge = rand.nextLong();

		HashSet<INodeID> receivers = new HashSet<>(3);
		receivers.add(INodeID.get("A"));
		receivers.add(INodeID.get("B"));
		receivers.add(INodeID.get("C"));

		SiSRequest req = new SiSRequest(timeout, maxInformationAge, SiSScope.SET, receivers, null);
		
		SiSInfoProperties info = new SiSInfoProperties();
		info.setScope(SiSScope.SET);
		info.setNodeId(INodeID.get("A"));

		assertTrue(req.satisfiableBy(info));

	}

	@Test
	public void test011_satisfiableBy() {

		long timeout = rand.nextLong();
		long maxInformationAge = rand.nextLong();

		HashSet<INodeID> receivers = new HashSet<>(3);
		receivers.add(INodeID.get("A"));
		receivers.add(INodeID.get("B"));
		receivers.add(INodeID.get("C"));

		SiSRequest req = new SiSRequest(timeout, maxInformationAge, SiSScope.SET, receivers, null);
		
		SiSInfoProperties info = new SiSInfoProperties();
		info.setScope(SiSScope.SET);
		info.setNodeId(INodeID.get("D"));

		assertFalse(req.satisfiableBy(info));

	}

	// TODO: Add more tests for satisfiableBy()

}
