/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of Simonstrator.KOM.
 * 
 * Simonstrator.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.sis.type.aggregation;

import java.util.Collection;

import de.tudarmstadt.maki.simonstrator.api.common.graph.BasicGraph;
import de.tudarmstadt.maki.simonstrator.api.common.graph.Graph;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSInformationConsumer.AggregationFunction;
import de.tudarmstadt.maki.simonstrator.api.component.sis.exception.AggregationNotPossibleException;
import de.tudarmstadt.maki.simonstrator.api.component.sis.type.SiSTypes;

/**
 * {@link AggregationFunction} for {@link Graph} in {@link SiSTypes}.
 * 
 * Using Set Theory (https://en.wikipedia.org/wiki/Set_theory)
 * 
 * @author Nils Richerzhagen
 *
 */
public class AggregationGraph extends AbstractAggregation<Graph> {

	/**
	 * Complete merge - full merge of edges and full merge of nodes.
	 */
	@Override
	protected Graph sum(Collection<Graph> data) throws AggregationNotPossibleException {
		assert data.size() > 0;
		if (data.size() == 1) {
			return data.iterator().next();
		}
		Graph aggregatedGraph = new BasicGraph();
		for (Graph graph : data) {
			aggregatedGraph.mergeGraph(graph);
		}
		return aggregatedGraph;
	}

	/**
	 * Only what is in all structures. <b>Intersection</b>. TODO!
	 */
	@Override
	protected Graph min(Collection<Graph> data) throws AggregationNotPossibleException {
		return super.min(data);
	}

	/**
	 * Same as SUM
	 */
	@Override
	protected Graph max(Collection<Graph> data) throws AggregationNotPossibleException {
		return sum(data);
	}

	/**
	 * Same as SUM
	 */
	@Override
	protected Graph avg(Collection<Graph> data) throws AggregationNotPossibleException {
		return sum(data);
	}

	@Override
	protected Graph count(Collection<Graph> data) throws AggregationNotPossibleException {
		// Currently not supported
		return super.count(data);
	}
}
