package de.tudarmstadt.maki.simonstrator.api.component.sis.util;

import java.util.Set;

import de.tudarmstadt.maki.simonstrator.api.common.graph.Graph;
import de.tudarmstadt.maki.simonstrator.api.common.graph.INodeID;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSComponent;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSDataCallback;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSInfoProperties;
import de.tudarmstadt.maki.simonstrator.api.component.sis.SiSInformationProvider.SiSProviderHandle;
import de.tudarmstadt.maki.simonstrator.api.component.sis.exception.InformationNotAvailableException;
import de.tudarmstadt.maki.simonstrator.api.component.sis.type.SiSType;
import de.tudarmstadt.maki.simonstrator.api.component.sis.type.SiSTypes;
import de.tudarmstadt.maki.simonstrator.api.component.topology.TopologyID;
import de.tudarmstadt.maki.simonstrator.api.component.topology.TopologyProvider;

/**
 * Utility-Bridge between {@link TopologyProvider} and {@link SiSComponent}.
 * 
 * Added 15.09.17 NR: Added the possibility for WiFi Underlay returning the
 * whole graph (SiSTypes.NEIGHBORS_WIFI) as before and additionally
 * SiSTypes.NEIGHBORS_WIFI_LOCAL which is returning the one hop neighborhood
 * graph of the requested node.
 * 
 * @author Bjoern Richerzhagen
 *
 */
public class SiSTopologyProvider implements SiSDataCallback<Graph> {

	private final TopologyProvider topo;

	private final TopologyID topoId;

	private Set<INodeID> localNodeID;

	private SiSInfoProperties infoProperties;

	private final SiSType<Graph> type;

	public SiSTopologyProvider(SiSComponent sis, SiSType<Graph> type,
			TopologyProvider topo, TopologyID topoId, Class<?> srcComponent) {
		this.topo = topo;
		this.topoId = topoId;
		this.type = type;
		this.localNodeID = INodeID.getSingleIDSet(sis.getHost().getId());
		infoProperties = new SiSInfoProperties()
				.setSourceComponent(srcComponent);
		sis.provide().nodeState(type, this);
	}

	@Override
	public SiSInfoProperties getInfoProperties() {
		return infoProperties;
	}

	@Override
	public Set<INodeID> getObservedNodes() {
		return localNodeID;
	}

	@Override
	public Graph getValue(INodeID nodeID, SiSProviderHandle providerHandle)
			throws InformationNotAvailableException {
		if (localNodeID.contains(nodeID)) {
			if (type == SiSTypes.NEIGHBORS_WIFI_LOCAL) {
				// Return the one hop neighborhood
				return topo.getLocalView(topoId).getLocalView(nodeID, 1);
			} else if (type == SiSTypes.NEIGHBORS_WIFI) {
				// return the complete underlay graph (GK)
				return topo.getLocalView(topoId);
			} else {
				throw new AssertionError("Unknown SiSType in SiSTopologyProvider " + type.toString());
			}
		}
		throw new InformationNotAvailableException();
	}

}
