/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.tudarmstadt.maki.simonstrator.api.component.vehicular.api.EdgeController;

public class RoadNetwork {
	private Map<String, RoadNetworkEdge> _roadNetwork;

	public RoadNetwork(SerializableRoadNetwork pRoadNetwork, EdgeController pEdgeController) {
		_roadNetwork = new HashMap<>();
		for (SerializableRoadNetworkEdge edge : pRoadNetwork.getAvailableEdges()) {
			_roadNetwork.put(edge.getEdgeID(), new RoadNetworkEdge(edge, pEdgeController));
		}

		for (SerializableRoadNetworkEdge edge : pRoadNetwork.getAvailableEdges()) {
			RoadNetworkEdge roadNetworkEdge = _roadNetwork.get(edge.getEdgeID());

			for (String edgeID : edge.getAccessibleEdges()) {
				roadNetworkEdge.addConnectedEdge(_roadNetwork.get(edgeID));
			}
		}
	}

	public RoadNetwork(Map<String, RoadNetworkEdge> pRoadNetwork) {
		_roadNetwork = pRoadNetwork;
	}

	public List<RoadNetworkEdge> getAvailableEdges() {
		return new ArrayList<>(_roadNetwork.values());
	}

	public RoadNetworkEdge getEdge(String pEdge) {
		return _roadNetwork.get(pEdge);
	}
}
