/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 *
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tudarmstadt.maki.simonstrator.api.component.vehicular.roadnetwork;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.tudarmstadt.maki.simonstrator.api.component.vehicular.api.EdgeController;

public class RoadNetworkEdge {
	private String _edgeID;
	private double _angle;

	private double _originalMaxSpeed = 0;

	private List<RoadNetworkLane> _lanes = new ArrayList<>();

	private List<RoadNetworkEdge> _accessibleEdges = new ArrayList<>();

	private final EdgeController _edgeController;

	public RoadNetworkEdge(SerializableRoadNetworkEdge pEdge, EdgeController pEdgeController) {
		_edgeController = pEdgeController;

		_edgeID = pEdge.getEdgeID();
		_angle = pEdge.getAngle();

		for (SerializableRoadNetworkLane lane : pEdge.getLanes()) {
			_lanes.add(new RoadNetworkLane(lane));
		}

		_originalMaxSpeed = pEdge.getMaxSpeed();
	}

	public RoadNetworkEdge(String pEdgeID, double pAngle, EdgeController pEdgeController) {
		_edgeController = pEdgeController;

		_edgeID = pEdgeID;
		_angle = pAngle;
	}

	public List<RoadNetworkLane> getLanes() {
		return _lanes;
	}

	public String getEdgeID() {
		return _edgeID;
	}

	public double getAngle() {
		return _angle;
	}

	public double getOriginalMaxSpeed() {
		return _originalMaxSpeed;
	}

	public int getLaneAmount() {
		return _lanes.size();
	}

	public void addLane(RoadNetworkLane pLane) {
		_lanes.add(pLane);
	}

	public void setOriginalMaxSpeed(double pMaxSpeed) {
		if (pMaxSpeed > _originalMaxSpeed) {
			_originalMaxSpeed = pMaxSpeed;
		}
	}

	public void addConnectedEdge(RoadNetworkEdge pEdge) {
		if (!_accessibleEdges.contains(pEdge)) {
			_accessibleEdges.add(pEdge);
		}
	}

	public List<RoadNetworkEdge> getAccessibleEdges() {
		return Collections.unmodifiableList(_accessibleEdges);
	}

	public String getDescription() {
		return "id: " + _edgeID + ", lanes: " + _lanes.size() + ", speed limit: " + _originalMaxSpeed;
	}

	@Override
	public String toString() {
		return _edgeID;
	}

	public boolean isInternal() {
		return _edgeID.startsWith(":");
	}

	public void changeEdgeMaxSpeed(double pMaxSpeed) {
		_edgeController.setEdgeMaxSpeed(getEdgeID(), pMaxSpeed);
	}
}
